;;;;Lisp Routine by Tony McLean Technical Records
(defun C:ListOut(/ *acad cur_dwg_lst dir fold progbar)
  (defun *error* (msg)
    (and Express ProgBar (acet-ui-progress))
    (or (wcmatch (strcase msg) "*BREAK,*CANCEL*,*EXIT*")
        (princ (strcat "\n** Error: " msg " **")))
    (princ))
  
  (setq *acad (cond (*acad) ((vlax-get-acad-object))))
  (setq Express
	     (and (vl-position "acetutil.arx" (arx))
		  (not
		    (vl-catch-all-error-p
		      (vl-catch-all-apply
			(function (lambda nil (acet-sys-shift-down))))))))
  (setq fold (getvar 'dwgprefix)  )
  (setq dir (DirDialog "Select folder to list out Xrefs of all drawings" nil 0))

  (setq cur_dwg_lst (vl-directory-files  dir "*.dwg")); acad_strlsort vl-sort
  (if (> (vl-list-length cur_dwg_lst) 0)
    (RunODBX cur_dwg_lst dir )
    (alert (strcat "Not found any drawing in folder: " dir
		   "\nPlease select other folder.") ))
  (princ))

(defun DirDialog (msg dir flag / Shell Fold Path)
    (setq *acad (cond (*acad) ((vlax-get-acad-object))))
    (setq Shell (vla-getInterfaceObject *acad "Shell.Application")
          Fold  (vlax-invoke-method Shell 'BrowseForFolder
                  (vla-get-HWND *acad) msg flag dir))
    (vlax-release-object Shell)
    (if Fold
      (progn
        (setq Path (vlax-get-property (vlax-get-property Fold 'Self) 'Path))
        (vlax-release-object Fold)
        (and (= "\\" (substr Path (strlen Path)))
             (setq Path (substr Path 1 (1- (strlen Path)))))))    
    Path)

(defun RunODBX (dwglst dir /  dbx dbxdoc doclst err flag lst progbar Opened )
  (defun ObjectDBXDocument (/ acVer)
    (setq *acad (cond (*acad) ((vlax-get-acad-object))))    
    (vla-GetInterfaceObject *acad
      (if (< (setq acVer (atoi (getvar "ACADVER"))) 16) "ObjectDBX.AxDbDocument"
        (strcat "ObjectDBX.AxDbDocument." (itoa acVer)))))
  (defun ObjRelease (lst)
    (mapcar
      (function
        (lambda (x)
          (if (and (eq (type x) 'VLA-OBJECT)
                   (not (vlax-object-released-p x)))
            (vl-catch-all-apply
              (function vlax-release-object) (list x))))) lst))
  (acet-ui-message "After you select the Directory, it may take a while before report is complete, please be paitent." "Alert" 64)
  (vlax-for doc (vla-get-Documents *acad)
    (setq DocLst
	   (cons
            (cons
              (strcase
                (vla-get-fullname doc)) doc) DocLst)))
  (setq dbxdoc (ObjectDBXDocument))
  (if Express (setq ProgBar (acet-ui-progress "Please wait ! Processing now ..." (length dwgLst))))
  (setq txtName (strcat dir "\\Xref_List.txt"))
  (setq	Opened (open txtName "w" ) )
  (write-line "----------------------------------------------------------------------" opened)
  (write-line "------------ NTG Technical Records Xref Audit LISP Routine 47351 ---------" opened  ) 
  (write-line (strcat "        Folder name: [" dir "]" ) opened  )
  (write-line (strcat "        Number of drawings: [" (itoa (vl-list-length cur_dwg_lst)) "]") opened)
  (write-line (strcat "        " (GetDate)) opened)
  (write-line "Ver. 291116" opened)
  (write-line "----------------------------------------------------------------------" opened)
  (write-line "Report: When a line is repeated with  Missing & Not Found this means its in the Paperspace not in an attributable block" opened  )
  (write-line "Report: Where -Pasted Image # appears as : Missing It is actually not missing - Ignore this" opened  ) 
  
  (foreach dwg dwgLst
    (if Express (acet-ui-progress -1))
    (setq dwg (strcat dir "\\" dwg)
	  flag (and (setq dbx (cdr (assoc (strcase dwg) DocLst)))))
    (and (not flag)
	 (setq Err (vl-catch-all-apply (function vla-open) (list dbxdoc dwg))
	       dbx dbxdoc))
    (princ (strcat "\n" dwg " : " ) )
    (write-line (strcat "\n" dwg " : " ) opened)
    (if (or flag (not (vl-catch-all-error-p Err)))
      (progn
;;;	(princ (strcat "\n" dwg " : " ) )
;;;	(write-line (strcat "\n" dwg " : " ) opened)
	(setq lst (ListOut_ODBX dbx dir))
	(if (> (vl-list-length lst) 0)
	  (foreach item (vl-sort lst '<)
;;;	    (princ (strcat "\n\t-" item))
	    (write-line (strcat "\t-" item) opened))
;;;	  (princ "\n\t-Not found any Xref in drawing." )
	  (write-line "\t-Not found any Xref in drawing." opened)
	  )
	)
      (progn
	(princ (strcat "\n** Error Opening File: " (vl-filename-base dwg)  ".dwg **"))
	(write-line (strcat "\t** Error Opening File: " (vl-filename-base dwg)  ".dwg **") opened))) )
  (if Opened    (close Opened)  )
  (ObjRelease (list dbx dbxdoc))
  (if Express (setq ProgBar (acet-ui-progress)))
  (if (findfile txtName)
    (startapp "notepad.exe" txtName)
    (textscr))
  (princ))

(defun ListOut_ODBX(source dir / res file found objname path imgs)
  (setq res (list))
  (vlax-for blk (vla-get-Blocks source)
    (if (= (vla-get-IsXref blk) :vlax-true)
      (progn
	(setq path (vla-get-path blk))
	(if (= "." (substr path 1 1))
	  (setq path (strcat dir (substr path 2))))
	(if (findfile path)
	  (setq found "OK")
	  (setq found "Missing") )
	(setq res (cons (strcat path ": " found ) res))	)    )  )

  (if (not(vl-catch-all-error-p (setq imgs
	(vl-catch-all-apply 'vla-item
	  (list (vla-get-dictionaries source) "ACAD_IMAGE_DICT")))))
    (vlax-for img imgs
      (setq path (cdr (assoc 1 (entget (vlax-vla-object->ename img) ) ) ))
      (if (= "." (substr path 1 1))
	(setq path (strcat dir (substr path 2))))
	(if (findfile path)
	  (setq found "OK")
	  (setq found "Missing") )
	(setq res (cons (strcat path ": " found ) res))  ) ) 
  
  (vlax-for lay (vla-get-layouts source)
    (vlax-for obj (vla-get-block lay)
      (setq file nil)
      (setq objName (vla-get-Objectname obj)	    )
      (cond
	((member objName '("AcDbRasterImage"))
	 (setq file (vla-get-imageFile obj)))
	((member objName '("AcDbPdfReference"))
	 (setq file (vla-get-File obj))	 )	) ;cond
      (if (/= file nil)
	(progn
	  (if (= "." (substr file 1 1))
	    (setq file (strcat dir (substr file 2))))
	  (if (findfile file)
	    (setq found "OK")
	    (setq found "Not Found") )
	  (setq res (cons (strcat file ": " found ) res))	  )      ) ))  
  res)

(defun GetDate (/ DATST MON DAY YEAR HRS)
  (setq DATST	(rtos (getvar "CDATE") 2 16)
	MON	(substr DATST 5 2)
	DAY	(substr DATST 7 2)
	YEAR	(substr DATST 1 4)
	HRS	(atoi (substr DATST 10 2))    )
  (cond ((= MON "01") (setq MON2 "January"))
	((= MON "02") (setq MON2 "Feburary"))
	((= MON "03") (setq MON2 "March"))
	((= MON "04") (setq MON2 "April"))
	((= MON "05") (setq MON2 "May"))
	((= MON "06") (setq MON2 "June"))
	((= MON "07") (setq MON2 "July"))
	((= MON "08") (setq MON2 "August"))
	((= MON "09") (setq MON2 "September"))
	((= MON "10") (setq MON2 "October"))
	((= MON "11") (setq MON2 "November"))
	((= MON "12") (setq MON2 "December"))    )
  (cond ((= HRS 00) (setq NHRS (itoa (+ HRS 12))) (setq XTR "AM"))
	((< HRS 12) (setq NHRS (itoa HRS)) (setq XTR "AM"))
	((= HRS 12) (setq NHRS (itoa HRS)) (setq XTR "PM"))
	((> HRS 12) (setq NHRS (itoa (- HRS 12))) (setq XTR "PM"))    )
  (strcat MON2 " " DAY  ", " YEAR " "
	  NHRS ":" (substr DATST 12 2) " " XTR )  )
(vl-load-com)
(princ  "\n Input < ListOut > to list out Xrefs of all drawings.")